SystemOrganization addCategory: #'Rest-Server-Handlers'!
SystemOrganization addCategory: #'Rest-Server-Models'!
SystemOrganization addCategory: #'Rest-Server-Services'!
SystemOrganization addCategory: #'Rest-Server'!
SystemOrganization addCategory: #'Rest-Server-Utilities'!

!HTTPResponse class methodsFor: '*rest-server' stamp: 'topa 5/20/2009 17:39'!
unauthorizedWithRealm: aRealmString

	| response |
	response := self unauthorized.
	response headers addField: (HTTPAuthorizationBasicRealmField new realm: aRealmString).
	^ response! !

Notification subclass: #RestDynamicVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Utilities'!

!RestDynamicVariable class methodsFor: 'defaults' stamp: 'topa 5/20/2009 16:28'!
defaultValue

	^ nil! !

!RestDynamicVariable class methodsFor: 'evaluating' stamp: 'topa 5/20/2009 16:29'!
use: anObject during: aBlock

	^ aBlock
		on: self
		do: [:notification | notification resume: anObject]! !

!RestDynamicVariable class methodsFor: 'evaluating' stamp: 'topa 5/20/2009 16:30'!
value

	^ self signal! !

!RestDynamicVariable methodsFor: 'defaults' stamp: 'topa 5/20/2009 16:27'!
defaultAction

	^ self class defaultValue! !

!HTTPPut methodsFor: '*rest-server' stamp: 'topa 4/3/2009 17:02'!
for: aRequestLine readFrom: aSwazooStream 

	super for: aRequestLine readFrom: aSwazooStream.
	self octetDataFrom: aSwazooStream.! !

Object subclass: #RestEntity
	instanceVariableNames: 'id'
	classVariableNames: 'LastId'
	poolDictionaries: ''
	category: 'Rest-Server-Models'!

!RestEntity class methodsFor: 'description' stamp: 'mt 5/6/2009 15:51'!
collectionPropertyNames

	^ #()! !

!RestEntity class methodsFor: 'accessing' stamp: 'cs 5/19/2009 18:24'!
handlerClass

	self subclassResponsibility! !

!RestEntity class methodsFor: 'private' stamp: 'mt 5/6/2009 15:52'!
incrementId

	self lastId: self lastId + 1.
	^ self lastId! !

!RestEntity class methodsFor: 'accessing' stamp: 'mt 5/6/2009 15:52'!
lastId

	^ LastId ifNil: [self lastId: 1. self lastId]! !

!RestEntity class methodsFor: 'accessing' stamp: 'mt 5/6/2009 15:52'!
lastId: anObject 

	LastId := anObject.! !

!RestEntity class methodsFor: 'description' stamp: 'mt 5/6/2009 15:52'!
primitivePropertyNames

	^ #(id restUri)! !

!RestEntity class methodsFor: 'description' stamp: 'mt 5/6/2009 15:51'!
referencePropertyNames

	^ #()! !

!RestEntity methodsFor: 'deleting' stamp: 'cs 5/20/2009 14:33'!
delete

	self subclassResponsibility! !

!RestEntity methodsFor: 'accessing' stamp: 'mt 5/6/2009 15:48'!
id

	^ id! !

!RestEntity methodsFor: 'accessing' stamp: 'mt 5/6/2009 15:48'!
id: anObject

	id := anObject.! !

!RestEntity methodsFor: 'printing' stamp: 'mt 5/6/2009 15:55'!
jsonWriteOn: aStream
	"Write its own properties onto the stream:
	'{'title'->'Just an entity', 
	  'ref project' -> '/projects/3', 
	  'col userStories' -> ['/..', '/..']}'
	'ref' indicates a reference to another resource
	'col' indicates a collection of other resources"

	^ (self responseDepth: 1) jsonWriteOn: aStream! !

!RestEntity methodsFor: 'printing' stamp: 'mt 5/6/2009 15:50'!
responseCollection: aSymbol depth: depth
	
	^ 'col ', aSymbol ->
		(depth < 1
			ifTrue: [self restUri, '/', aSymbol]
			ifFalse: [((self perform: aSymbol asSymbol)
				collect: [:each | each responseDepth: depth - 1])])! !

!RestEntity methodsFor: 'printing' stamp: 'mt 5/6/2009 15:55'!
responseDepth: depth
	"Returns a Dictionary with the properties. If the properties are entities and the depth is higher than 1 then resolve recursivly."

	| result |
	depth = 0 ifTrue: [^ self restUri].
	result := Dictionary newFrom: (self class primitivePropertyNames collect: [ :aSymbol |
			aSymbol -> (self perform: aSymbol)]).
	result addAll: (Dictionary newFrom: (self class referencePropertyNames collect: [ :aSymbol | 
			'ref ', aSymbol -> ((self perform: aSymbol)
				ifNotNilDo: [:entity | entity responseDepth: depth - 1])])).
	result addAll: (Dictionary newFrom: (self class collectionPropertyNames collect: [ :aSymbol |
			self responseCollection: aSymbol depth: depth - 1])).
	^ result! !

!RestEntity methodsFor: 'accessing' stamp: 'cs 5/19/2009 18:23'!
restUri

	^ (self class handlerClass uriMapping at: self class) value: self! !

!RestEntity methodsFor: 'accessing' stamp: 'mt 5/6/2009 15:56'!
restUri: anObject
	"Read-only."

	^ self! !

Object subclass: #RestHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestHttpHandler class methodsFor: 'testing' stamp: 'cs 5/19/2009 15:12'!
canBeAutostarted

	^ false! !

!RestHttpHandler class methodsFor: 'handling' stamp: 'cs 5/22/2009 12:15'!
pathPattern

	^ String new! !

!RestHttpHandler class methodsFor: 'as yet unclassified' stamp: 'topa 5/20/2009 17:44'!
realm
	"Default Realm, if Authentication is used"
	
	^ 'REST'! !

!RestHttpHandler methodsFor: 'request processing' stamp: 'topa 5/20/2009 19:41'!
handle: anHTTPRequest
	"Handles an http request. 
	That is: process the request with errorhandling."

	^ self 
		handleAuthentication: anHTTPRequest
		continueWith: [[self process: anHTTPRequest] 
			on: RestError
			do: [:error | error asHTTPResponse]]! !

!RestHttpHandler methodsFor: 'request processing' stamp: 'cs 5/22/2009 13:48'!
handleAuthentication: anHTTPRequest continueWith: aBlock
	"Check, whether authentication was requested and continue upon successful auth."
	
	| authHeader |
	^ self needsAuthentication 
		ifFalse: aBlock
		ifTrue: [
			authHeader := anHTTPRequest headers 
					fieldNamed: 'Authorization' 
					ifNone: [^ HTTPResponse unauthorizedWithRealm: self class realm].
			(self isAuthenticated: authHeader)
					ifFalse: [HTTPResponse unauthorized]
					ifTrue: aBlock]! !

!RestHttpHandler methodsFor: 'as yet unclassified' stamp: 'topa 5/20/2009 17:18'!
isAuthenticated: anHeaderField

	^ true! !

!RestHttpHandler methodsFor: 'authentication' stamp: 'topa 3/5/2009 11:29'!
needsAuthentication
	"override, if HTTPAuth is needed"
	
	^ false! !

!RestHttpHandler methodsFor: 'request processing' stamp: 'cs 5/22/2009 14:39'!
pathComponents: anHTTPRequest

	^ anHTTPRequest uri identifierPath allButFirst:
		1 + (self class pathPattern findTokens: '/') size! !

!RestHttpHandler methodsFor: 'request processing' stamp: 'topa 5/20/2009 17:22'!
process: anHTTPRequest
	"Handles an http request. Should answer with an http response."
	
	self subclassResponsibility.! !

RestHttpHandler subclass: #RestResourceHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestResourceHttpHandler class methodsFor: 'error handling' stamp: 'cs 3/18/2009 15:29'!
responseForFailure

	^ HTTPResponse internalServerError! !

!RestResourceHttpHandler class methodsFor: 'resources' stamp: 'cs 5/19/2009 09:15'!
rootResources

	self subclassResponsibility! !

!RestResourceHttpHandler class methodsFor: 'resources' stamp: 'cs 5/19/2009 09:16'!
uriMapping

	self subclassResponsibility! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'cs 5/22/2009 16:12'!
process: anHTTPRequest 

	^ (self processorForRequest: anHTTPRequest) 
			process: (self rootComponent: anHTTPRequest)
			components: (self pathComponents: anHTTPRequest) allButFirst! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'topa 3/24/2009 15:32'!
processorForRequest: anHTTPRequest

	| processorClass |
	anHTTPRequest isGet ifTrue: [processorClass := RestGetRequestProcessor].
	anHTTPRequest isPost ifTrue: [processorClass := RestPostRequestProcessor].
	anHTTPRequest isDelete ifTrue: [processorClass := RestDeleteRequestProcessor].
	anHTTPRequest isPut ifTrue: [processorClass := RestPutRequestProcessor].

	^ processorClass "dispatch if not found"
		ifNil: [RestNotImplementedError signal: 'This method is not implemented by Rest']
		ifNotNil: [processorClass newFrom: anHTTPRequest handler: self]! !

!RestResourceHttpHandler methodsFor: 'authorization' stamp: 'SR 4/21/2009 13:40'!
requestAllowed: aRequest for: anObject

	^ true! !

!RestResourceHttpHandler methodsFor: 'parsing' stamp: 'cs 5/22/2009 16:12'!
rootComponent: anHTTPRequest
	"Return an association between the resource name and the resource class for the root component according to the HTTP request."

	^ self
		rootComponent: anHTTPRequest
		ifAbsent: [RestResolveError signal: 'Requested root component not found.']! !

!RestResourceHttpHandler methodsFor: 'parsing' stamp: 'mt 3/2/2009 15:03'!
rootComponent: anHTTPRequest ifAbsent: aBlock
	"Return an association between the resource name and the resource class for the root component according to the HTTP request."

	^ self class rootResources
		associationAt: ((self pathComponents: anHTTPRequest) at: 1 ifAbsent: aBlock)
		ifAbsent: aBlock! !

!RestResourceHttpHandler methodsFor: 'authorization' stamp: 'mt 5/6/2009 16:02'!
selectorAllowed: aSelector for: aRestEntity

	^ aRestEntity class referencePropertyNames,
		aRestEntity class collectionPropertyNames 
			includes: aSelector! !

Object subclass: #RestRequestProcessor
	instanceVariableNames: 'request handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

RestRequestProcessor subclass: #RestDeleteRequestProcessor
	instanceVariableNames: 'postData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestDeleteRequestProcessor methodsFor: 'processing' stamp: 'cs 5/22/2009 15:40'!
responseBodyForRoot: root components: components
	"Do recursive entity resolving. Start with root. Return an empty body due to DELETE request."
	
	| entity parentEntity selector |
	entity := self resolveId: root value all components: components.
	'/', ((self handler pathComponents: request) join: '/') = entity restUri
		ifTrue: [
			(entity respondsTo: #delete)
				ifTrue: [entity perform: #delete]
				ifFalse: [RestResolveError signal: 'Selector delete not found.']]
		ifFalse: [
			parentEntity := self resolveId: root value all components: components allButLast allButLast.
			selector := self selectorFor: components allButLast last.
			(parentEntity perform: selector with: entity)].
	^ String new! !

!RestDeleteRequestProcessor methodsFor: 'processing' stamp: 'topa 5/25/2009 11:28'!
selectorFor: resourceName
	"Return the selector for adding a resource depending on the given resource name."
	
	| selector |
	selector := 'remove', resourceName capitalized.
	(selector endsWith: 'ies') ifTrue: [selector := (selector allButLast: 3), 'y'].
	(selector endsWith: 's') ifTrue: [selector := selector allButLast].
	selector := (selector, ':') asSymbol.
	^ selector! !

RestRequestProcessor subclass: #RestGetRequestProcessor
	instanceVariableNames: 'depth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestGetRequestProcessor class methodsFor: 'as yet unclassified' stamp: 'cs 4/27/2009 17:43'!
newFrom: anHTTPRequest handler: aHandler

	^ (super newFrom: anHTTPRequest handler: aHandler)
		depth: ((anHTTPRequest queryAt: 'depth') ifNotNilDo: [:n | n asNumber]);
		yourself! !

!RestGetRequestProcessor methodsFor: 'accessing' stamp: 'cs 4/27/2009 17:37'!
depth

	(depth isNil or: [depth < 0 or: [depth > 4]]) ifTrue: [depth := 1].
	^ depth! !

!RestGetRequestProcessor methodsFor: 'accessing' stamp: 'cs 4/27/2009 17:37'!
depth: anObject

	depth := anObject.! !

!RestGetRequestProcessor methodsFor: 'processing' stamp: 'cs 4/27/2009 11:34'!
resolveMethod: anEntity components: pathComponents
	
	| selector resource |
	pathComponents ifEmpty: [^ anEntity].
	
	selector := (pathComponents first endsWith: 'WithDeleted') | (pathComponents size = 1)
		ifTrue: [pathComponents first asSymbol]
		ifFalse: [(pathComponents first, 'WithDeleted') asSymbol].
	(self handler selectorAllowed: selector for: anEntity)  
		ifTrue: [resource := anEntity perform: selector]
		ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})].
	
	self handler requestAllowed: self request for: resource.

	^ (pathComponents size = 2 and: [pathComponents last = 'all'])
		ifTrue: [resource]
		ifFalse: [self resolveId: resource components: pathComponents allButFirst]! !

!RestGetRequestProcessor methodsFor: 'processing' stamp: 'cs 4/27/2009 17:38'!
responseBodyForRoot: root components: pathComponents
	"Do recursive entity resolving. Start with root."
		
	| entity |
	pathComponents isEmpty
		ifTrue: [entity := self resolveId: root value all components: pathComponents]
		ifFalse: [entity := self resolveId: root value allWithDeleted components: pathComponents].

	^ Json render: (entity isCollection
		ifTrue: [entity collect: [:e | e responseDepth: self depth - 1]]
		ifFalse: [entity responseDepth: self depth])! !

RestRequestProcessor subclass: #RestPostRequestProcessor
	instanceVariableNames: 'postData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestPostRequestProcessor methodsFor: 'adding' stamp: 'mt 3/2/2009 15:50'!
addResource: aResourceName to: anEntity
	"Resolve the selector and perform an add-operation on the given entity with the postData."
	
	| selector |
	selector := self selectorFor: aResourceName.
	
	(anEntity respondsTo: selector)
		ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})].
		
	^ (anEntity perform: selector with: self postData) restUri
! !

!RestPostRequestProcessor methodsFor: 'private' stamp: 'mig 3/13/2009 12:24'!
postData
	"Data could be JSON string with properties or a simple string with a rest uri (not JSON). 
	The first one will be parsed into a dictionary. The last one will be resolved to the real entity."

	postData ifNil: [ | data |
		self request class category = 'Hyper' ifFalse: [self request ensureFullRead].
		data := Json readFrom: self request entityBody value readStream.
		postData := data isString 
			ifTrue: [self resolveUri: data]
			ifFalse: [self parseData: data].
	].
	^ postData! !

!RestPostRequestProcessor methodsFor: 'resolving' stamp: 'mfp 3/6/2009 14:49'!
resolveMethod: anEntity components: pathComponents
	"New recursion abort condition. Perform add-operation due to POST request (instead of returning the entity)."
	
	^ pathComponents size = 1
		ifTrue: [
			self handler requestAllowed: self request for: anEntity.
			self addResource: pathComponents first to: anEntity]
		ifFalse: [
			super resolveMethod: anEntity components: pathComponents]! !

!RestPostRequestProcessor methodsFor: 'processing' stamp: 'mt 3/2/2009 15:51'!
responseBodyForRoot: root components: components
	"Do recursive entity resolving. Start with root."
	
	^ components isEmpty
		ifTrue: [self addResource: root key to: root value]
		ifFalse: [self resolveId: root value all components: components]! !

!RestPostRequestProcessor methodsFor: 'adding' stamp: 'cs 5/20/2009 13:55'!
selectorFor: resourceName
	"Return the selector for adding a resource depending on the given resource name."
	
	| selector |
	selector := self postData isCollection
		ifTrue: ['addNew', resourceName capitalized]
		ifFalse: ['add', resourceName capitalized].
	(selector endsWith: 'ies') ifTrue: [selector := (selector allButLast: 3), 'y'].
	(selector endsWith: 's') ifTrue: [selector := selector allButLast].
	selector := (selector, ':') asSymbol.
	^ selector! !

RestRequestProcessor subclass: #RestPutRequestProcessor
	instanceVariableNames: 'putData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestPutRequestProcessor methodsFor: 'private' stamp: 'mt 3/2/2009 16:00'!
putData

	putData ifNil: [
		putData := Json readFrom: (HTTPString stringFromBytes: self request putData)
			value "convert bytes to string"
			readStream "convert string to readstream"].
	^ putData! !

!RestPutRequestProcessor methodsFor: 'processing' stamp: 'mt 3/16/2009 13:01'!
responseBodyForRoot: root components: pathComponents
	"Do recursive entity resolving. Start with root."
	
	| entity selector |
	entity := self resolveId: root value all components: pathComponents.
	(self parseData: self putData) keysAndValuesDo: [:key :value | 
		selector := key asSymbol asMutator.
		(((entity class primitivePropertyNames union: entity class referencePropertyNames) includes: key asSymbol) and:
			[entity respondsTo: selector])
				ifTrue: [entity perform: selector with: value]
				ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})]].
	^ entity restUri! !

!RestRequestProcessor class methodsFor: 'instance creation' stamp: 'cs 2/23/2009 18:06'!
newFrom: anHTTPRequest handler: aHandler
	^ self new
		request: anHTTPRequest;
		handler: aHandler! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:57'!
handler

	^ handler! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:57'!
handler: anObject

	handler := anObject! !

!RestRequestProcessor methodsFor: 'helping' stamp: 'cs 3/18/2009 15:26'!
parseData: aDictionary

	| newDict |
	newDict := Dictionary new.
	aDictionary keysAndValuesDo: [:key :value | newDict add: (
		(key startsWith: 'ref ')
			ifTrue: [(key allButFirst: 4) -> (value ifNotNil: [self resolveUri: value])]
			ifFalse: [key -> value])].
	^ newDict! !

!RestRequestProcessor methodsFor: 'processing' stamp: 'topa 5/25/2009 12:32'!
process: root components: pathComponents
	"The root should be an association between name and class of the root resource."

	^ HTTPResponse ok
		contentType: 'application/json; charset=iso-8859-15';
		entity: (self responseBodyForRoot: root components: pathComponents);
		yourself
! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:57'!
request

	^ request! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:58'!
request: anObject

	request := anObject! !

!RestRequestProcessor methodsFor: 'resolving' stamp: 'SR 3/9/2009 13:11'!
resolveId: aCollection components: pathComponents
	"Try to resolve the next element in the path to a number (id). This id should be a valid index in the given collection. Hint: Recursive."

	| id resource |
	pathComponents ifEmpty: [^ aCollection].
	
	id := pathComponents first asNumber.
	resource := aCollection 
		detect: [:each | each id = id] 
		ifNone: [RestResolveError signal: ('Entity with id {1} not found.' format: {id})].

	self handler requestAllowed: self request for: resource.

	^ self resolveMethod: resource components: pathComponents allButFirst! !

!RestRequestProcessor methodsFor: 'resolving' stamp: 'mig 3/25/2009 16:38'!
resolveMethod: anEntity components: pathComponents
	"Try to resolve the next element in the path to a method. This method/message should be understood by the entity. Hint: Recursive."
	
	| selector resource |
	pathComponents ifEmpty: [^ anEntity].
	
	selector := pathComponents first asSymbol.
	(self handler selectorAllowed: selector for: anEntity)  
		ifTrue: [resource := anEntity perform: selector]
		ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})].
	
	self handler requestAllowed: self request for: resource.

	^ self resolveId: resource components: pathComponents allButFirst! !

!RestRequestProcessor methodsFor: 'helping' stamp: 'mfp 3/6/2009 15:53'!
resolveUri: aString
	"Finds an object with the provided uri by using an pseudo-get-request. This is the only reason why we need a reference to the handler."

	|component components|
	components := aString findTokens: '/'.
	component :=  self handler class rootResources 
		at: components first 
		ifAbsent:[RestResolveError signal: 'invalid URI given'].

	^ self 
		resolveId: component all
		components: components allButFirst! !

!RestRequestProcessor methodsFor: 'processing' stamp: 'mt 3/2/2009 15:14'!
responseBodyForRoot: root components: pathComponents

	self subclassResponsibility.! !

!Collection methodsFor: '*rest-server' stamp: 'cs 5/22/2009 15:39'!
join: delimter
	"expects self to be collection of strings and delimter to be a string.
	returns the concatenation of the elements separeted by the delimiter"

	| stream |
	stream := WriteStream with: String new.
	self
		do: [: elem | stream nextPutAll: elem]
		separatedBy: [ stream nextPutAll: delimter].
	^ stream contents! !

!Collection methodsFor: '*rest-server' stamp: 'topa 2/25/2009 13:25'!
jsonWriteOn: aStream

	aStream nextPut: $[.
	self
		do: [ :each | each jsonWriteOn: aStream]
		separatedBy: [aStream nextPut: $,].
	aStream nextPut: $].! !

Resource subclass: #RestHttpService
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Services'!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService commentStamp: 'mt 2/16/2009 14:56' prior: 0!
I am a restful http service. I am primarily used for the XP integration platform project. (Package: 'Xpf')!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService class methodsFor: 'service' stamp: 'cs 5/22/2009 14:48'!
autostartHandlers

	^ RestHttpHandler allSubclasses
		select: [:handler | handler canBeAutostarted]! !

!RestHttpService class methodsFor: 'error handling' stamp: 'topa 05/19/2009 12:46'!
handleError: anError

	(Smalltalk includesKey: #System) 
		ifTrue: [self handleErrorGemstone: anError]
		ifFalse: [self handleErrorDefault: anError].
	
	^ HTTPResponse internalServerError entity: anError description! !

!RestHttpService class methodsFor: 'error handling' stamp: 'cs 5/22/2009 15:20'!
handleErrorDefault: anError

	anError signal.
	Transcript show: '[EE]: ', anError description.! !

!RestHttpService class methodsFor: 'error handling' stamp: 'topa 05/12/2009 16:59'!
handleErrorGemstone: anError

	System abortTransaction.
	GsFile gciLogServer: '[EE]: ', anError description.! !

!RestHttpService class methodsFor: 'service' stamp: 'cs 5/22/2009 12:17'!
installHandlers

	self autostartHandlers do: [:handler | 
		self resource installHandler: handler]! !

!RestHttpService class methodsFor: 'testing' stamp: 'topa 3/3/2009 21:11'!
isRunning

	^ self site notNil and: [site isServing]! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:11'!
port

	^ self site port! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:12'!
resource

	^ resource! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:12'!
resource: anObject

	resource := anObject.! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:12'!
site

	^ site! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:13'!
site: anObject

	site := anObject.! !

!RestHttpService class methodsFor: 'service' stamp: 'cs 5/19/2009 09:08'!
start

	self startOn: 8082.
	self installHandlers.! !

!RestHttpService class methodsFor: 'service' stamp: 'cs 5/19/2009 15:24'!
startOn: aPort
	"Start exactly one http service. Does nothing if server is already running."

	self isRunning ifTrue: [^ self].
	self site: (Site new
		name: 'rest';
		host: '*' port: aPort).

	SwazooServer singleton addSite: self site.
	self resource: (self new uriPattern: '/').
	self site addResource: self resource.
	SwazooServer singleton startSite: 'rest'.! !

!RestHttpService class methodsFor: 'service' stamp: 'topa 3/3/2009 21:15'!
stop
	"Shutdown the server. Does nothing if server is not running."

	self site ifNotNil:
		[self site stop.
		SwazooServer singleton removeSite: self site.
		self site: nil; resource: nil].! !

!RestHttpService methodsFor: 'serving' stamp: 'topa 5/20/2009 17:03'!
answerTo: request
	
	^ (self handlerFor: request)
		ifNil: [HTTPResponse notFound]
		ifNotNilDo: [:handler |
			[handler value new handle: request]
				on: Error 
				do: [:error | self class handleError: error]]! !

!RestHttpService methodsFor: 'serving' stamp: 'cs 5/22/2009 15:20'!
handlerFor: aRequest

	| uri |
	uri := aRequest uri asString copyReplaceAll: '//' with: '/'.
	^ self handlers
		detect: [:ass| uri startsWith: ass key]
		ifNone: [nil]! !

!RestHttpService methodsFor: 'accessing' stamp: 'cs 5/19/2009 15:02'!
handlers

	^ handlers! !

!RestHttpService methodsFor: 'accessing' stamp: 'cs 5/19/2009 15:02'!
handlers: anObject

	handlers := anObject.! !

!RestHttpService methodsFor: 'initialization' stamp: 'cs 5/19/2009 15:06'!
initialize

	super initialize.
	self handlers: (SortedCollection sortBlock: [:firstAssoc :secondAssoc |
		firstAssoc key size > secondAssoc key size]).! !

!RestHttpService methodsFor: 'handlers' stamp: 'cs 5/22/2009 14:24'!
installHandler: aRequestHandlerClass

	self installHandler: aRequestHandlerClass on: '/', aRequestHandlerClass pathPattern! !

!RestHttpService methodsFor: 'handlers' stamp: 'cs 5/19/2009 15:04'!
installHandler: aRequestHandlerClass on: aPathString

	(self handlers anySatisfy: [:ass | ass key = aPathString ])
		ifTrue: [self error: (
			'Handler with path {1} already installed' format: {aPathString})].
	self handlers add: aPathString -> aRequestHandlerClass.! !

!RestHttpService methodsFor: 'handlers' stamp: 'cs 5/22/2009 14:56'!
removeHandler: aRequestHandlerClass
	
	self handlers remove:
		(self handlers detect: [:assoc | assoc value = aRequestHandlerClass] ifNone: [^ self]).! !

SpecificHeaderField subclass: #HTTPAuthorizationBasicRealmField
	instanceVariableNames: 'realm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!HTTPAuthorizationBasicRealmField class methodsFor: 'as yet unclassified' stamp: 'SR 5/13/2009 11:21'!
fieldName

	^ 'WWW-Authenticate'! !

!HTTPAuthorizationBasicRealmField methodsFor: 'accessing' stamp: 'SR 5/13/2009 11:02'!
realm

	^ realm! !

!HTTPAuthorizationBasicRealmField methodsFor: 'accessing' stamp: 'SR 5/13/2009 11:02'!
realm: anObject

	realm := anObject! !

!HTTPAuthorizationBasicRealmField methodsFor: 'as yet unclassified' stamp: 'topa 5/19/2009 13:38'!
valuesAsStringOn: aStream 
	
	(Smalltalk includesKey: #System)
		ifTrue: [aStream
			nextPutAllCHARs: 'Basic realm="';
			nextPutAllCHARs: self realm;
			nextPutCHAR: $"]
		ifFalse: [aStream
			nextPutAll: 'Basic realm="';
			nextPutAll: self realm;
			nextPutAll: '"']! !

Error subclass: #RestError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

RestError subclass: #RestNotAllowedError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestNotAllowedError methodsFor: 'as yet unclassified' stamp: 'mfp 3/6/2009 11:42'!
asHTTPResponse

	^ HTTPResponse forbidden entity: self messageText! !

RestError subclass: #RestNotImplementedError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestNotImplementedError methodsFor: 'as yet unclassified' stamp: 'topa 3/24/2009 15:29'!
asHTTPResponse

	^ HTTPResponse notImplemented entity: self messageText! !

RestError subclass: #RestResolveError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestResolveError methodsFor: 'as yet unclassified' stamp: 'topa 3/3/2009 21:02'!
asHTTPResponse

	^ HTTPResponse notFound entity: self messageText! !
